package sample;

import java.util.ArrayList;
import java.util.List;

import xmlgp.Experiment;
import xmlgp.Generation;
import xmlgp.Individual;
import xmlgp.Node;
import xmlgp.Project;
import xmlgp.Representation;
import xmlgp.Statistics;
import xmlgp.Treatment;
import xmlgp.Trial;
import xmlgp.XMLGP;

import static xmlgp.NodeType.NON_TERMINAL;
import static xmlgp.NodeType.TERMINAL;
import static xmlgp.RepresentationType.TREE;

/**
 * This class is to show how to use XMLGP tool.<p>
 * This class has a method to show how to construct project object 
 * and serializes that object to an XML file.
 * This class also has a method to show how to deserialze XML file to project object 
 * and prints its members.
 * @author dharani
 *
 */
public class Sample {

	/**
	 * Constructs the project object and serializes to an XML file.
	 * @param fileName XML file name
	 */
	private void createXMLOutput(String fileName) 
	{
		try
		{
			//constructs project object
			Project project = formXML();
			//uses the XMLGP library to serialize the object to a file 
			XMLGP.serializeToFile(project, fileName);
		}		
		catch(Exception e) 
		{
			e.printStackTrace();
		} 
	}
	
	/**
	 * Constructs project object by using the setters in XMLGP library.
	 * @return Project - the holder for the project information
	 */
	private Project formXML() 
	{
		Project project = null;

		try 
		{	
			//Just for an example statistics is set only to project			
			Statistics statistics = new Statistics();
			statistics.setName("Mean");
			statistics.setValue("2.2");
			
			ArrayList<Statistics> statList = new ArrayList<Statistics>();
			statList.add(statistics);

			Node lnode = new Node();
			lnode.setLeafNode(TERMINAL, "1");

			Node rnode = new Node();			
			rnode.setLeafNode(TERMINAL, "1");
			
			ArrayList<Node> children = new ArrayList<Node>();
			children.add(lnode);
			children.add(rnode);

			Node node = new Node();			
			node.setNonTerminalNode(NON_TERMINAL, "+", children);

			Representation representation = new Representation();			
			representation.setAttributes(1, TREE, null, node);
			
			List<Representation> representations = new ArrayList<Representation>();
			representations.add(representation);

			Individual individual = new Individual();
			individual.setAttributes(1, null, representations);
			
			List<Individual> individuals = new ArrayList<Individual>();
			individuals.add(individual);

			Generation generation = new Generation();			
			generation.setAttributes(1, null, individuals);
			
			List<Generation> generations = new ArrayList<Generation>();
			generations.add(generation);

			Trial trial = new Trial();			
			trial.setAttributes(1, null, null, generations);
			
			List<Trial> trials = new ArrayList<Trial>();
			trials.add(trial);

			Treatment treatment = new Treatment();			
			treatment.setAttributes(1, "Test treatment", null, trials);
			
			List<Treatment> treatments = new ArrayList<Treatment>();
			treatments.add(treatment);

			Experiment experiment = new Experiment();			
			experiment.setAttributes(1, "19th June", "Test Exp", null, treatments);
			
			List<Experiment> experiments = new ArrayList<Experiment>();
			experiments.add(experiment);

			project = new Project();			
			project.setAttributes(1, "XML GP test", "Test Class", "19th June", "SC Lab", "Ten million Won", statList, experiments);
			
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		}

		return project;
	}
	
	
	/**
	 * Constructs project object by using the setters in XMLGP library.
	 * This method is just to show how to set attributes of the objects one by one.
	 * @return Project - the holder for the project information
	 */
	private Project formXMLOneByOne() 
	{
		Project project = null;

		try 
		{	
			//Just for an example statistics is set only to project			
			Statistics statistics = new Statistics();
			statistics.setName("Mean");
			statistics.setValue("2.2");
			
			ArrayList<Statistics> statList = new ArrayList<Statistics>();
			statList.add(statistics);

			Node lnode = new Node();
			lnode.setType(TERMINAL);
			lnode.setValue("1");
			
			Node rnode = new Node();
			rnode.setType(TERMINAL);
			rnode.setValue("1");
					
			ArrayList<Node> children = new ArrayList<Node>();
			children.add(lnode);
			children.add(rnode);

			Node node = new Node();
			node.setType(NON_TERMINAL);
			node.setValue("+");
			node.setChildren(children);
			
			Representation representation = new Representation();		
			representation.setType(TREE);
			representation.setNode(node);
			representation.setId(1);
						
			List<Representation> representations = new ArrayList<Representation>();
			representations.add(representation);

			Individual individual = new Individual();			
			individual.setRepresentations(representations);
			individual.setId(1);
			
			List<Individual> individuals = new ArrayList<Individual>();
			individuals.add(individual);

			Generation generation = new Generation();			
			generation.setIndividuals(individuals);
			generation.setId(1);	
			
			List<Generation> generations = new ArrayList<Generation>();
			generations.add(generation);

			Trial trial = new Trial();			
			trial.setGenerations(generations);
			trial.setId(1);	
		
			List<Trial> trials = new ArrayList<Trial>();
			trials.add(trial);

			Treatment treatment = new Treatment();		
			treatment.setTrials(trials);
			treatment.setId(1);		
			
			List<Treatment> treatments = new ArrayList<Treatment>();
			treatments.add(treatment);

			Experiment experiment = new Experiment();			
			experiment.setTreatments(treatments);
			experiment.setId(1);
			experiment.setDescription("Test Exp");
			experiment.setDate("19th June");
			
			List<Experiment> experiments = new ArrayList<Experiment>();
			experiments.add(experiment);

			project = new Project();
			project.setExperiments(experiments);
			project.setId(1);
			project.setName("XML GP test");
			project.setDescription("Test Class");
			project.setDate("19th June");
			project.setOrganization("SC Lab");
			project.setFund("Ten million Won");
			project.setStatistics(statList);
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		}

		return project;
	}

	/**
	 * Deserializes XML to Project object and prints its members.
	 * @param fileName the XML file name
	 */
	private void processProject(String fileName) 
	{
		try 
		{
			//uses the XMLGP library to deserialize XML to project object
			Project project = XMLGP.deserializeFromFile(fileName);
			//uses the XMLGP library to print the contents of the project object
			XMLGP.printGPResults(project);
		}		
		catch (Exception e) 
		{
			e.printStackTrace();
		} 		
	}
	
	/**
	 * This is the main method for sample class to show how to use XMLGP.
	 * @param args XML file name
	 */
	public static void main(String[] args) 
	{
		Sample createXML = new Sample();
		//constructs project object and serializes that object to an XML file
		createXML.createXMLOutput("test.xml");
		//deserialzes the given XML file to project object and prints its members
		createXML.processProject("test.xml");
	}
}
